using System;
using Extensibility;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio.CommandBars;
using System.Resources;
using System.Reflection;
using System.Globalization;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Stripem
{
	/// <summary>The object for implementing an Add-in.</summary>
	/// <seealso class='IDTExtensibility2' />
	public sealed partial class Connect : IDTExtensibility2, IDTCommandTarget
	{
		private DTE2 _applicationObject;
		private AddIn _addInInstance;

        private StripemOptions.EolStyle _style = StripemOptions.EolStyle.Uinitialized;
        private bool _enableFilenameFilter = false;
        private string _filenameFilter = ".*\\.(txt|cpp|c|h|hpp|msg|idl)$";

		/// <summary>Implements the constructor for the Add-in object. Place your initialization code within this method.</summary>
		public Connect()
		{
		}

		/// <summary>Implements the OnConnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being loaded.</summary>
		/// <param term='application'>Root object of the host application.</param>
		/// <param term='connectMode'>Describes how the Add-in is being loaded.</param>
		/// <param term='addInInst'>Object representing this Add-in.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnConnection(object application, ext_ConnectMode connectMode, object addInInst, ref Array custom)
		{
			_applicationObject = (DTE2)application;
			_addInInstance = (AddIn)addInInst;
			if(connectMode == ext_ConnectMode.ext_cm_UISetup)
			{
				object []contextGUIDS = new object[] { };
				Commands2 commands = (Commands2)_applicationObject.Commands;
				string toolsMenuName;

				try
				{
					//If you would like to move the command to a different menu, change the word "Tools" to the 
					//  English version of the menu. This code will take the culture, append on the name of the menu
					//  then add the command to that menu. You can find a list of all the top-level menus in the file
					//  CommandBar.resx.
					string resourceName;
					ResourceManager resourceManager = new ResourceManager("Stripem.CommandBar", Assembly.GetExecutingAssembly());
					CultureInfo cultureInfo = new CultureInfo(_applicationObject.LocaleID);
					
					if(cultureInfo.TwoLetterISOLanguageName == "zh")
					{
						System.Globalization.CultureInfo parentCultureInfo = cultureInfo.Parent;
						resourceName = String.Concat(parentCultureInfo.Name, "Tools");
					}
					else
					{
						resourceName = String.Concat(cultureInfo.TwoLetterISOLanguageName, "Tools");
					}
					toolsMenuName = resourceManager.GetString(resourceName);
				}
				catch
				{
					//We tried to find a localized version of the word Tools, but one was not found.
					//  Default to the en-US word, which may work for the current culture.
					toolsMenuName = "Tools";
				}

				//Place the command on the tools menu.
				//Find the MenuBar command bar, which is the top-level command bar holding all the main menu items:
				Microsoft.VisualStudio.CommandBars.CommandBar menuBarCommandBar = ((Microsoft.VisualStudio.CommandBars.CommandBars)_applicationObject.CommandBars)["MenuBar"];

				//Find the Tools command bar on the MenuBar command bar:
				CommandBarControl toolsControl = menuBarCommandBar.Controls[toolsMenuName];
				CommandBarPopup toolsPopup = (CommandBarPopup)toolsControl;

                // Find the standard command bar
                Microsoft.VisualStudio.CommandBars.CommandBar stdCmdBar = ((Microsoft.VisualStudio.CommandBars.CommandBars)_applicationObject.CommandBars)["Standard"];

				//This try/catch block can be duplicated if you wish to add multiple commands to be handled by your Add-in,
				//  just make sure you also update the QueryStatus/Exec method to include the new command names.
				try
				{
					//Add a command to the Commands collection:
					Command command = commands.AddNamedCommand2(_addInInstance, "Stripem", "Strip'em", "Set line ending style",
                                                                false, 1, ref contextGUIDS,
                                                                (int)vsCommandStatus.vsCommandStatusSupported+(int)vsCommandStatus.vsCommandStatusEnabled,
                                                                (int)vsCommandStyle.vsCommandStylePictAndText, vsCommandControlType.vsCommandControlTypeButton);

					if(command != null)
					{
    					//Add a control for the command to the tools menu:
                        if (toolsPopup != null)
                        {
                            CommandBarButton cmdBarBtn = (CommandBarButton)command.AddControl(toolsPopup.CommandBar, 1);
                            cmdBarBtn.Style = MsoButtonStyle.msoButtonIconAndCaption;
                        }

                        // Add a button to the standard command bar
                        if (stdCmdBar != null)
                        {
                            CommandBarButton cmdBarBtn = (CommandBarButton)command.AddControl(stdCmdBar, stdCmdBar.Controls.Count + 1);
                            //stdCmdBarCtl.Caption = "Set line ending style";
                            //CommandBarButton cmdBarBtn = (CommandBarButton)stdCmdBarCtl;
                            cmdBarBtn.Style = MsoButtonStyle.msoButtonIcon;
                        }
                    }

                }
				catch(System.ArgumentException)
				{
					//If we are here, then the exception is probably because a command with that name
					//  already exists. If so there is no need to recreate the command and we can 
                    //  safely ignore the exception.
				}
			}

            // Read the last used style from the registry
            if (_style == StripemOptions.EolStyle.Uinitialized)
            {
                // Attempt to open the setting key
                RegistryKey key = Registry.CurrentUser.OpenSubKey("Software\\Grebulon\\Stripem2008", false);
                // If the key doesn't exist, create it
                if (key == null)
                {
                    //System.Windows.Forms.MessageBox.Show("Creating registry key \"Software\\Grebulon\\Stripem2008\"", "Strip'em", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Information);
                    key = Registry.CurrentUser.CreateSubKey("Software\\Grebulon\\Stripem2008");
                }

                // Get the style setting
                if (key != null)
                {
                    _style = (StripemOptions.EolStyle)(int)key.GetValue("EolStyle", StripemOptions.EolStyle.Disabled);
                    _enableFilenameFilter = (int)key.GetValue("EnableFilter", 0) != 0;
                    _filenameFilter = (string)key.GetValue("FilenameFilter", ".*\\.(txt|cpp|c|h|hpp|msg|idl)$");
                    //System.Windows.Forms.MessageBox.Show("Got registry style: " + _style + "=" + (int)_style + "\nEnable=" + _enableFilenameFilter + ", Filter="+_filenameFilter, "Strip'em", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Information);
                }
                else
                {
                    _style = StripemOptions.EolStyle.Disabled;
                    System.Windows.Forms.MessageBox.Show("Failed to open registry key \"Software\\Grebulon\\Stripem2008\"", "Strip'em", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
                }
            }

            // Begin catching document events
            TrackDocumentEvents();
        }

		/// <summary>Implements the OnDisconnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being unloaded.</summary>
		/// <param term='disconnectMode'>Describes how the Add-in is being unloaded.</param>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnDisconnection(ext_DisconnectMode disconnectMode, ref Array custom)
		{
            ReleaseDocumentEvents();
		}

		/// <summary>Implements the OnAddInsUpdate method of the IDTExtensibility2 interface. Receives notification when the collection of Add-ins has changed.</summary>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />		
		public void OnAddInsUpdate(ref Array custom)
		{
		}

		/// <summary>Implements the OnStartupComplete method of the IDTExtensibility2 interface. Receives notification that the host application has completed loading.</summary>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnStartupComplete(ref Array custom)
		{
		}

		/// <summary>Implements the OnBeginShutdown method of the IDTExtensibility2 interface. Receives notification that the host application is being unloaded.</summary>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnBeginShutdown(ref Array custom)
		{
		}
		
		/// <summary>Implements the QueryStatus method of the IDTCommandTarget interface. This is called when the command's availability is updated</summary>
		/// <param term='commandName'>The name of the command to determine state for.</param>
		/// <param term='neededText'>Text that is needed for the command.</param>
		/// <param term='status'>The state of the command in the user interface.</param>
		/// <param term='commandText'>Text requested by the neededText parameter.</param>
		/// <seealso class='Exec' />
		public void QueryStatus(string commandName, vsCommandStatusTextWanted neededText, ref vsCommandStatus status, ref object commandText)
		{
			if(neededText == vsCommandStatusTextWanted.vsCommandStatusTextWantedNone)
			{
				if(commandName == "Stripem.Connect.Stripem")
				{
					status = (vsCommandStatus)vsCommandStatus.vsCommandStatusSupported|vsCommandStatus.vsCommandStatusEnabled;
					return;
				}
			}
		}

		/// <summary>Implements the Exec method of the IDTCommandTarget interface. This is called when the command is invoked.</summary>
		/// <param term='commandName'>The name of the command to execute.</param>
		/// <param term='executeOption'>Describes how the command should be run.</param>
		/// <param term='varIn'>Parameters passed from the caller to the command handler.</param>
		/// <param term='varOut'>Parameters passed from the command handler to the caller.</param>
		/// <param term='handled'>Informs the caller if the command was handled or not.</param>
		/// <seealso class='Exec' />
		public void Exec(string commandName, vsCommandExecOption executeOption, ref object varIn, ref object varOut, ref bool handled)
		{
			if(executeOption == vsCommandExecOption.vsCommandExecOptionDoDefault)
			{
				if(commandName == "Stripem.Connect.Stripem")
				{
                    StripemOptions optDlg = new StripemOptions(_style, _enableFilenameFilter, _filenameFilter);
                    if (optDlg.ShowDialog() == DialogResult.OK)
                    {
                        if (_style != optDlg.Style || _enableFilenameFilter != optDlg.EnableFilenameFilter ||
                            _filenameFilter != optDlg.FilenameFilter)
                        {
                            _style = optDlg.Style;
                            _enableFilenameFilter = optDlg.EnableFilenameFilter;
                            _filenameFilter = optDlg.FilenameFilter;

                            // Write the style to the registry
                            // Attempt to open the setting key
                            RegistryKey key = Registry.CurrentUser.OpenSubKey("Software\\Grebulon\\Stripem2008", true);
                            if (key != null)
                            {
                                key.SetValue("EolStyle", (int)_style, RegistryValueKind.DWord);
                                key.SetValue("EnableFilter", _enableFilenameFilter ? 1 : 0);
                                key.SetValue("FilenameFilter", _filenameFilter);
                                //System.Windows.Forms.MessageBox.Show("save registry style: " + _style + "=" + (int)_style + "\nEnable=" + _enableFilenameFilter + ", Filter=" + _filenameFilter, "Strip'em", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Information);
                            }
                            else
                                System.Windows.Forms.MessageBox.Show("Failed to open registry key \"Software\\Grebulon\\Stripem2008\"", "Strip'em", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
                        }
                    }
				}
			}
		}
	}
}